@@ -2,7 +2,7 @@
 #[cfg(test)]
 mod test;
 
-use crate::sync::{Ordering, AtomicUsize};
+use crate::sync::{Ordering};
 use crate::sync::{Mutex, Arc};
 use crate::sync::{SpinMutex};
 
@@ -19,6 +19,7 @@ use crate::dynamic_chunk::{DynamicChunkRecycled};
 use crate::{StartPositionEpoch};
 
 /// This way you can control when chunk's memory deallocation happens.
+/// _In addition, some operations may cause deallocations as well._
 #[derive(PartialEq)]
 pub enum CleanupMode{
     /// Cleanup will be called when chunk fully read.
@@ -29,7 +30,7 @@ pub enum CleanupMode{
     OnChunkRead,
     /// Cleanup will be called when new chunk created.
     OnNewChunk,
-    /// Cleanup will never be called. You should call [EventQueue::cleanup] manually.
+    /// Cleanup will never be called. You should call `EventQueue::cleanup` manually.
     Never
 }
 
@@ -38,6 +39,7 @@ pub trait Settings{
     const MAX_CHUNK_SIZE : u32;
     const CLEANUP        : CleanupMode;
 
+    // for spmc/mpmc
     /// Lock on new chunk cleanup event. Will dead-lock if already locked.
     const LOCK_ON_NEW_CHUNK_CLEANUP: bool;
     /// Call cleanup on unsubscribe?
@@ -48,6 +50,9 @@ pub struct List<T, S: Settings>{
     first: *mut DynamicChunk<T, S>,
     last : *mut DynamicChunk<T, S>,
     chunk_id_counter: usize,
+    total_capacity: usize,
+
+    readers_count: u32,
 
     /// 0 - means no penult
     penult_chunk_size: u32,
@@ -60,13 +65,11 @@ pub struct List<T, S: Settings>{
 pub struct EventQueue<T, S: Settings>{
     pub(crate) list  : Mutex<List<T, S>>,
 
-    /// All atomic op relaxed. Just to speed up `try_clean` check (opportunistic check).
-    /// Mutated under list lock.
-    pub(crate) readers: AtomicUsize,
-
     /// Separate lock from list::start_position_epoch, is safe, because start_point_epoch encoded in
     /// chunk's atomic len+epoch.
-    pub(crate) start_position: SpinMutex<Cursor<T, S>>,
+    // TODO: Make RWLock? Bench.
+    // TODO: Optioned
+    pub(crate) start_position: SpinMutex<Option<Cursor<T, S>>>,
 
     _pinned: PhantomPinned,
 }
@@ -84,14 +87,14 @@ impl<T, S: Settings> EventQueue<T, S>
                 first: null_mut(),
                 last: null_mut(),
                 chunk_id_counter: 0,
-
+                readers_count:0,
+                total_capacity:new_capacity as usize,
                 penult_chunk_size : 0,
 
                 #[cfg(feature = "double_buffering")]
                 free_chunk: None,
             }),
-            readers : AtomicUsize::new(0),
-            start_position: SpinMutex::new(Cursor{chunk: null(), index:0}),
+            start_position: SpinMutex::new(None),
             _pinned: PhantomPinned,
         });
 
@@ -102,7 +105,6 @@ impl<T, S: Settings> EventQueue<T, S>
             let event = &mut *(&*this as *const _ as *mut EventQueue<T, S>);
             event.list.get_mut().first = node;
             event.list.get_mut().last  = node;
-            event.start_position.get_mut().chunk = node;
         }
 
         this
@@ -152,6 +154,7 @@ impl<T, S: Settings> EventQueue<T, S>
         node.set_next(new_node, Ordering::Release);
         list.last = new_node;
         list.penult_chunk_size = node.capacity() as u32;
+        list.total_capacity += size;
 
         unsafe{&mut *new_node}
     }
@@ -161,8 +164,8 @@ impl<T, S: Settings> EventQueue<T, S>
         if S::CLEANUP == CleanupMode::OnNewChunk{
             // this should acts as compile-time-if.
             if S::LOCK_ON_NEW_CHUNK_CLEANUP{
-                // `cleanup` - locks internally
-                self.cleanup();
+                let _lock = self.list.lock();
+                self.cleanup_impl(list);
             } else {
                 self.cleanup_impl(list);
             }
@@ -242,30 +245,22 @@ impl<T, S: Settings> EventQueue<T, S>
     /// EventReader will start receive events from NOW.
     /// It will not see events that was pushed BEFORE subscription.
     pub fn subscribe(&self, list: &mut List<T, S>) -> EventReader<T, S>{
-        let prev_readers = self.readers.fetch_add(1, Ordering::Relaxed);
-        if prev_readers == 0{
+        if list.readers_count == 0{
             // Keep alive. Decrements in unsubscribe
             unsafe { Arc::increment_strong_count(self); }
         }
+        list.readers_count += 1;
 
         let last_chunk = unsafe{&*list.last};
         let chunk_state = last_chunk.chunk_state(Ordering::Relaxed);
-        let last_chunk_len = chunk_state.len();
-        let epoch = chunk_state.epoch();
 
-        let mut event_reader = EventReader{
-            position: Cursor{chunk: list.first, index: 0},
-            start_position_epoch: epoch
-        };
+        // Enter chunk
+        last_chunk.readers_entered().fetch_add(1, Ordering::AcqRel);
 
-        // Move to an end. This will increment read_completely_times in all passed chunks correctly.
-        event_reader.set_forward_position(
-            Cursor{
-                chunk: last_chunk,
-                index: last_chunk_len as usize
-            },
-            false);
-        event_reader
+        EventReader{
+            position: Cursor{chunk: last_chunk, index: chunk_state.len() as usize},
+            start_position_epoch: chunk_state.epoch()
+        }
     }
 
     // Called from EventReader Drop
@@ -276,36 +271,40 @@ impl<T, S: Settings> EventQueue<T, S>
         let this = unsafe { this_ptr.as_ref() };
         let mut list = this.list.lock();
 
-        // -1 read_completely_times for each chunk that reader passed
-        unsafe {
-            foreach_chunk(
-                list.first,
-                event_reader.position.chunk,
-                Ordering::Relaxed,      // we're under mutex
-                |chunk| {
-                    debug_assert!(
-                        !chunk.next(Ordering::Acquire).is_null()
-                    );
-                    chunk.read_completely_times().fetch_sub(1, Ordering::AcqRel);
-                    Continue(())
-                }
-            );
-        }
-
-        let prev_readers = this.readers.fetch_sub(1, Ordering::Relaxed);
+        // Exit chunk
+        unsafe{&*event_reader.position.chunk}.read_completely_times().fetch_add(1, Ordering::AcqRel);
 
         if S::CLEANUP_IN_UNSUBSCRIBE && S::CLEANUP != CleanupMode::Never{
-            this.cleanup_impl(&mut *list);
+            if list.first as *const _ == event_reader.position.chunk {
+                this.cleanup_impl(&mut *list);
+            }
         }
 
-        drop(list);
-        if prev_readers == 1{
+        list.readers_count -= 1;
+        if list.readers_count == 0{
+            drop(list);
             // Safe to self-destruct
             unsafe { Arc::decrement_strong_count(this_ptr.as_ptr()); }
         }
     }
 
-    unsafe fn free_chunk(chunk: *mut DynamicChunk<T, S>, list: &mut List<T, S>){
+    unsafe fn free_chunk<const LOCK_ON_WRITE_START_POSITION: bool>(
+        &self,
+        chunk: *mut DynamicChunk<T, S>,
+        list: &mut List<T, S>)
+    {
+        if let Some(start_position) = *self.start_position.as_mut_ptr(){
+            if start_position.chunk == chunk{
+                if LOCK_ON_WRITE_START_POSITION{
+                    *self.start_position.lock() = None;
+                } else {
+                    *self.start_position.as_mut_ptr() = None;
+                }
+            }
+        }
+
+        list.total_capacity -= (*chunk).capacity();
+
         #[cfg(not(feature = "double_buffering"))]
         {
             DynamicChunk::destruct(chunk);
@@ -327,7 +326,6 @@ impl<T, S: Settings> EventQueue<T, S>
     }
 
     fn cleanup_impl(&self, list: &mut List<T, S>){
-        let readers_count = self.readers.load(Ordering::Relaxed);
         unsafe {
             // using _ptr version, because with &chunk - reference should be valid during whole
             // lambda function call. (according to miri and some rust borrowing rules).
@@ -337,8 +335,12 @@ impl<T, S: Settings> EventQueue<T, S>
                 list.last,
                 Ordering::Relaxed,      // we're under mutex
                 |chunk_ptr| {
+                    // Do not lock prev_chunk.chunk_switch_mutex because we traverse in order.
                     let chunk = &mut *chunk_ptr;
-                    if chunk.read_completely_times().load(Ordering::Acquire) != readers_count{
+                    let chunk_readers = chunk.readers_entered().load(Ordering::Acquire);
+                    let chunk_read_times = chunk.read_completely_times().load(Ordering::Acquire);
+                    // Cleanup only in order
+                    if chunk_readers != chunk_read_times {
                         return Break(());
                     }
 
@@ -346,7 +348,9 @@ impl<T, S: Settings> EventQueue<T, S>
                     debug_assert!(!next_chunk_ptr.is_null());
 
                     debug_assert!(std::ptr::eq(chunk, list.first));
-                    Self::free_chunk(chunk, list);
+                    // Do not lock start_position permanently, because reader will
+                    // never enter chunk before list.first
+                    self.free_chunk::<true>(chunk, list);
                     list.first = next_chunk_ptr;
 
                     Continue(())
@@ -358,6 +362,55 @@ impl<T, S: Settings> EventQueue<T, S>
         }
     }
 
+    /// This will traverse up to the start_point - and will free all unoccupied chunks. (out-of-order cleanup)
+    /// This one slower then cleanup_impl.
+    fn force_cleanup_impl(&self, list: &mut List<T, S>){
+        self.cleanup_impl(list);
+
+        // Lock start_position permanently, due to out of order chunk destruction.
+        // Reader can try enter in the chunk in the middle of force_cleanup execution.
+        let start_position = self.start_position.lock();
+        let terminal_chunk = match &*start_position{
+            None => { return; }
+            Some(cursor) => {cursor.chunk}
+        };
+        if list.first as *const _ == terminal_chunk{
+            return;
+        }
+        unsafe {
+            // cleanup_impl dealt with first chunk before. Omit.
+            let mut prev_chunk = list.first;
+            // using _ptr version, because with &chunk - reference should be valid during whole
+            // lambda function call. (according to miri and some rust borrowing rules).
+            // And we actually drop that chunk.
+            foreach_chunk_ptr_mut(
+                (*list.first).next(Ordering::Relaxed),
+                terminal_chunk,
+                Ordering::Relaxed,      // we're under mutex
+                |chunk| {
+                    // We need to lock only `prev_chunk`, because it is impossible
+                    // to get in `chunk` omitting chunk.readers_entered+1
+                    let lock = (*prev_chunk).chunk_switch_mutex().write();
+                        let chunk_readers = (*chunk).readers_entered().load(Ordering::Acquire);
+                        let chunk_read_times = (*chunk).read_completely_times().load(Ordering::Acquire);
+                        if chunk_readers != chunk_read_times {
+                            prev_chunk = chunk;
+                            return Continue(());
+                        }
+
+                        let next_chunk_ptr = (*chunk).next(Ordering::Relaxed);
+                        debug_assert!(!next_chunk_ptr.is_null());
+
+                        (*prev_chunk).set_next(next_chunk_ptr, Ordering::Release);
+                    drop(lock);
+
+                    self.free_chunk::<false>(chunk, list);
+                    Continue(())
+                }
+            );
+        }
+    }
+
     pub fn cleanup(&self){
         self.cleanup_impl(&mut *self.list.lock());
     }
@@ -368,7 +421,7 @@ impl<T, S: Settings> EventQueue<T, S>
         list: &mut List<T, S>,
         new_start_position: Cursor<T, S>)
     {
-        *self.start_position.lock() = new_start_position;
+        *self.start_position.lock() = Some(new_start_position);
 
         // update len_and_start_position_epoch in each chunk
         let first_chunk = unsafe{&mut *list.first};
@@ -388,36 +441,39 @@ impl<T, S: Settings> EventQueue<T, S>
 
     pub fn clear(&self, list: &mut List<T, S>){
         let last_chunk = unsafe{ &*list.last };
-        let chunk_len = last_chunk.chunk_state(Ordering::Relaxed).len() as usize;
+        let last_chunk_len = last_chunk.chunk_state(Ordering::Relaxed).len() as usize;
 
         self.set_start_position(list, Cursor {
             chunk: last_chunk,
-            index: chunk_len
+            index: last_chunk_len
         });
+
+        self.force_cleanup_impl(list);
     }
 
     pub fn truncate_front(&self, list: &mut List<T, S>, len: usize) {
         // make chunks* array
-        let chunks_count= unsafe {
-            list.chunk_id_counter/*(*list.last).id*/ - (*list.first).id() + 1
-        };
 
+        // TODO: subtract from total_capacity
+        // TODO: use small_vec
+        // TODO: loop if > 128?
         // there is no way we can have memory enough to hold > 2^64 bytes.
-        debug_assert!(chunks_count<=128);
         let mut chunks : [*const DynamicChunk<T, S>; 128] = [null(); 128];
-        unsafe {
-            let mut i = 0;
-            foreach_chunk(
-                list.first,
-                null(),
-                Ordering::Relaxed,      // we're under mutex
-                |chunk| {
-                    chunks[i] = chunk;
-                    i+=1;
-                    Continue(())
-                }
-            );
-        }
+        let chunks_count=
+            unsafe {
+                let mut i = 0;
+                foreach_chunk(
+                    list.first,
+                    null(),
+                    Ordering::Relaxed,      // we're under mutex
+                    |chunk| {
+                        chunks[i] = chunk;
+                        i+=1;
+                        Continue(())
+                    }
+                );
+                i
+            };
 
         let mut total_len = 0;
         for i in (0..chunks_count).rev(){
@@ -425,10 +481,19 @@ impl<T, S: Settings> EventQueue<T, S>
             let chunk_len = chunk.chunk_state(Ordering::Relaxed).len() as usize;
             total_len += chunk_len;
             if total_len >= len{
-                self.set_start_position(list, Cursor {
+                let new_start_position = Cursor {
                     chunk: chunks[i],
                     index: total_len - len
-                });
+                };
+                // Do we actually need to truncate?
+                if let Some(start_position) = unsafe{*self.start_position.as_mut_ptr()}{
+                    if start_position >= new_start_position{
+                        return;
+                    }
+                }
+
+                self.set_start_position(list, new_start_position);
+                self.force_cleanup_impl(list);
                 return;
             }
         }
@@ -444,19 +509,7 @@ impl<T, S: Settings> EventQueue<T, S>
     }
 
     pub fn total_capacity(&self, list: &List<T, S>) -> usize {
-        let mut total = 0;
-        unsafe {
-            foreach_chunk(
-                list.first,
-                null(),
-                Ordering::Relaxed,      // we're under mutex
-                |chunk| {
-                    total += chunk.capacity();
-                    Continue(())
-                }
-            );
-        }
-        total
+        list.total_capacity
     }
 
     pub fn chunk_capacity(&self, list: &List<T, S>) -> usize {
@@ -476,7 +529,7 @@ impl<T, S: Settings> EventQueue<T, S>
 impl<T, S: Settings> Drop for EventQueue<T, S>{
     fn drop(&mut self) {
         let list = self.list.get_mut();
-        debug_assert!(self.readers.load(Ordering::Relaxed) == 0);
+        debug_assert!(list.readers_count == 0);
         unsafe{
             let mut node_ptr = list.first;
             while node_ptr != null_mut() {
@@ -540,7 +593,13 @@ pub(super) unsafe fn foreach_chunk_ptr_mut<T, F, S: Settings>
     debug_assert!(
         end_chunk_ptr.is_null()
             ||
-        std::ptr::eq((*start_chunk_ptr).event(), (*end_chunk_ptr).event()));
+        std::ptr::eq((*start_chunk_ptr).event(), (*end_chunk_ptr).event())
+    );
+    debug_assert!(
+        end_chunk_ptr.is_null()
+            ||
+        (*start_chunk_ptr).id() <= (*end_chunk_ptr).id()
+    );
 
     let mut chunk_ptr = start_chunk_ptr;
     while !chunk_ptr.is_null(){
